/*
 * radio_hal_interface.c
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <dlfcn.h>
#include <dlog.h>

#include "radio_hal_interface.h"

#ifdef LOG_TAG
#undef LOG_TAG
#endif /* LOG_TAG */

#define LOG_TAG "RADIO_HAL_INTF"

#define LIB_TIZEN_RADIO PATH_LIBDIR"/libtizen-radio.so"

#define RADIO_INTF_CHECK_ARG(x_radio)					\
do {												\
	if (!x_radio) {									\
		LOGE("argument is NULL\n");			\
		return RADIO_ERROR_INVALID_PARAMETER;	\
	}												\
} while (0)

int radio_hal_interface_init(radio_hal_interface **handle)
{
	radio_hal_interface *h = NULL;
	int ret = RADIO_ERROR_NONE;
	h = (radio_hal_interface *)malloc(sizeof(radio_hal_interface));

	if (!h) {
		LOGE("cannot allocate memory for radio_hal interface");
		return RADIO_ERROR_OUT_OF_MEMORY;
	}

	h->dl_handle = dlopen(LIB_TIZEN_RADIO, RTLD_NOW);

	if (h->dl_handle) {
		h->intf.init = dlsym(h->dl_handle, "radio_init");
		h->intf.deinit = dlsym(h->dl_handle, "radio_deinit");
		h->intf.prepare = dlsym(h->dl_handle, "radio_prepare");
		h->intf.unprepare = dlsym(h->dl_handle, "radio_unprepare");
		h->intf.open = dlsym(h->dl_handle, "radio_open");
		h->intf.close = dlsym(h->dl_handle, "radio_close");
		h->intf.start = dlsym(h->dl_handle, "radio_start");
		h->intf.stop = dlsym(h->dl_handle, "radio_stop");
		h->intf.seek = dlsym(h->dl_handle, "radio_seek");
		h->intf.get_frequency = dlsym(h->dl_handle, "radio_get_frequency");
		h->intf.set_frequency = dlsym(h->dl_handle, "radio_set_frequency");
		h->intf.mute = dlsym(h->dl_handle, "radio_mute");
		h->intf.unmute = dlsym(h->dl_handle, "radio_unmute");
		h->intf.get_signal_strength = dlsym(h->dl_handle, "radio_get_signal_strength");
		h->intf.get_volume = dlsym(h->dl_handle, "radio_get_volume");
		h->intf.set_volume = dlsym(h->dl_handle, "radio_set_volume");
		h->intf.set_media_volume = dlsym(h->dl_handle, "radio_set_media_volume");

		if (h->intf.init == NULL || h->intf.deinit == NULL) {
			LOGE("could not get mandatory funtion");
			goto FAIL;
		}

		ret = h->intf.init(&h->rh_handle);
		if (ret != RADIO_ERROR_NONE) {
			LOGE("radio_hal init failed %d", ret);
			goto FAIL;
		}

	} else {
		LOGE("open radio hal_interface failed : %s", dlerror());
		ret = RADIO_ERROR_INTERNAL;
		goto FAIL;
	}

	*handle = h;

	LOGD("open radio_hal interface");

	return ret;
FAIL:
	if (h) {
		if (h->dl_handle)
			dlclose(h->dl_handle);
		free(h);
	}

	return ret;
}

int radio_hal_interface_deinit(radio_hal_interface *handle)
{
	int ret = RADIO_ERROR_NONE;
	RADIO_INTF_CHECK_ARG(handle);

	if (handle->dl_handle) {
		ret = handle->intf.deinit(handle->rh_handle);
		if (ret != RADIO_ERROR_NONE)
			return ret;

		handle->rh_handle = NULL;

		LOGD("close radio_hal interface");
		dlclose(handle->dl_handle);
		handle->dl_handle = NULL;
	}

	free(handle);
	handle = NULL;

	return ret;
}

int radio_hal_open(radio_hal_interface *handle)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.open) {
		LOGW("radio_hal open is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.open(handle->rh_handle);
}

int radio_hal_prepare(radio_hal_interface *handle)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.prepare) {
		LOGW("radio_hal prepare is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.prepare(handle->rh_handle);
}

int radio_hal_unprepare(radio_hal_interface *handle)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.unprepare) {
		LOGW("radio_hal unprepare is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.unprepare(handle->rh_handle);
}


int radio_hal_close(radio_hal_interface *handle)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.close) {
		LOGW("radio_hal close is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.close(handle->rh_handle);
}

int radio_hal_start(radio_hal_interface *handle)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.start) {
		LOGW("radio_hal start is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.start(handle->rh_handle);
}

int radio_hal_stop(radio_hal_interface *handle)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.stop) {
		LOGW("radio_hal stop is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.stop(handle->rh_handle);
}


int radio_hal_seek(radio_hal_interface *handle, seek_direction_type_t direction)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.seek) {
		LOGW("radio_hal seek is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.seek(handle->rh_handle, (radio_seek_direction_type_t)direction);
}

int radio_hal_get_frequency(radio_hal_interface *handle, uint32_t *frequency)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.get_frequency) {
		LOGW("radio_hal get_frequency is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.get_frequency(handle->rh_handle, frequency);
}

int radio_hal_set_frequency(radio_hal_interface *handle, uint32_t frequency)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.set_frequency) {
		LOGW("radio_hal set_frequency is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.set_frequency(handle->rh_handle, frequency);
}

int radio_hal_get_signal_strength(radio_hal_interface *handle, int32_t *strength)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.get_signal_strength) {
		LOGW("radio_hal get_signal_strength is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.get_signal_strength(handle->rh_handle, strength);
}

int radio_hal_mute(radio_hal_interface *handle)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.mute) {
		LOGW("radio_hal mute is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.mute(handle->rh_handle);
}

int radio_hal_unmute(radio_hal_interface *handle)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.unmute) {
		LOGW("radio_hal unmute is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.unmute(handle->rh_handle);
}

int radio_hal_set_volume(radio_hal_interface *handle, float volume)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.set_volume) {
		LOGW("radio_hal set_volume is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.set_volume(handle->rh_handle, volume);
}

int radio_hal_get_volume(radio_hal_interface *handle, float *volume)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.get_volume) {
		LOGW("radio_hal get_volume is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.get_volume(handle->rh_handle, volume);
}

int radio_hal_set_media_volume(radio_hal_interface *handle, uint32_t level)
{
	RADIO_INTF_CHECK_ARG(handle);

	if (!handle->intf.set_media_volume) {
		LOGW("radio_hal set_media_volume is NULL");
		return RADIO_ERROR_NOT_IMPLEMENTED;
	}

	return handle->intf.set_media_volume(handle->rh_handle, level);
}

#ifdef __cplusplus
}
#endif /* __cplusplus */

